# SPDX-License-Identifier: CC0-1.0

"""
Haketilo unit tests - base
"""

# This file is part of Haketilo
#
# Copyright (C) 2021, Wojtek Kosior
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the CC0 1.0 Universal License as published by
# the Creative Commons Corporation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# CC0 1.0 Universal License for more details.

import pytest

from ..script_loader import load_script
from ..extension_crafting import ExtraHTML

def test_driver(driver):
    """
    A trivial test case that verifies mocked web pages served by proxy can be
    accessed by the browser driven.
    """
    for proto in ['http://', 'https://']:
        driver.get(proto + 'gotmyowndoma.in')
        title = driver.execute_script(
            'return document.getElementsByTagName("title")[0].innerText;'
        )
        assert "Schrodinger's Document" in title

@pytest.mark.get_page('https://gotmyowndoma.in')
def test_script_loader(execute_in_page):
    """
    A trivial test case that verifies Haketilo's .js files can be properly
    loaded into a test page together with their dependencies.
    """
    execute_in_page(load_script('common/indexeddb.js'))

    assert 'mapping' in execute_in_page('returnval(stores.map(s => s[0]));')

@pytest.mark.ext_data({})
@pytest.mark.usefixtures('webextension')
def test_webextension(driver):
    """
    A trivial test case that verifies a test WebExtension created and installed
    by the `webextension` fixture works and redirects specially-constructed URLs
    to its test page.
    """
    heading = driver.execute_script(
        'return document.getElementsByTagName("h1")[0].innerText;'
    )
    assert "Extension's options page for testing" in heading

@pytest.mark.ext_data({
    'extra_html': ExtraHTML(
        'html/default_blocking_policy.html',
        {
            'html/default_blocking_policy.js':
            'document.body.innerHTML = `ski-ba-bop-ba ${typeof by_id}`;'
        }
    ),
    'navigate_to': 'html/default_blocking_policy.html'
})
@pytest.mark.usefixtures('webextension')
def test_extra_html(driver):
    """
    A trivial test case of the facility for loading the Haketilo's HTML files
    into test WebExtension for unit-testing.
    """
    assert driver.execute_script('return document.body.innerText') == \
        'ski-ba-bop-ba function'
